#!/bin/sh


# High Level Activity Reporter
# Version 0.11.0 dev
# USE HLAR AT YOUR OWN RISK
#
# schedule script to cron
#
#0,10,20,30,40,50 * * * * /var/adm/hlar/hlar.sh -a collect -m ps,netstat-an,df-k,vmstat_page,vmstat,vmstat_vmm,vxstat,vxmemstat,vxfsstat_cache,vxfsstat_shared,vxfsstat_per_fs,mpstat,prstat-m,lockstat,who,ipcs,iostat -p 600 -i 60 >> /var/adm/hlar/hlar.log 2>&1
#56 23 * * * /var/adm/hlar/hlar.sh -a clean >> /var/adm/hlar/hlar.log 2>&1


#
# independent functions
#

isNumeric() {
  echo "$@" | egrep '^[0-9]+$' > /dev/null
}

not() { "$@"; [ $? != 0 ]; }

echoerr() { echo "$@" 1>&2; }

# functions usage and echoerr must exist 
fatalerror() {
  echoerr "ERROR: $@"
  usage
  exit 1
}

warning() {
  echoerr "WARNING: $@"
}

# dependent on the global variable DEBUG
debug() {
  [ "$DEBUG" = "true" ] && echoerr "DEBUG: $@"
}

#
# script specialized functions
#

usage() {
  SCRIPTNAME=`basename $0`
  echoerr "USAGE: $SCRIPTNAME -a collect -m <module,..> [-p <period>] [-i <interval>] [-o <outputdir>] [-d]"
  echoerr "USAGE: $SCRIPTNAME -a clean [-o <outputdir>] [-k <keep data>] [-d]"
  echoerr "USAGE:     -m : ps,netstat-an,df-k,vmstat_page,vmstat,vmstat_vmm,vxstat,vxmemstat,vxfsstat_cache,vxfsstat_shared,vxfsstat_per_fs,mpstat,prstat-m,lockstat,who,ipcs"
  echoerr "USAGE:     -p : period between $SCRIPTNAME runs (from cron) in seconds (default=600)"
  echoerr "USAGE:     -i : interval in seconds (default=10)"
  echoerr "USAGE:     -o : output directory (default=/var/adm/hlar)"
  echoerr "USAGE:     -k : keep data N days (default=7)"
  echoerr "USAGE:     -d : debug mode"
  echoerr "USAGE: example: $SCRIPTNAME -a collect -m netstat-an,df-k,vmstat,vmstat_page,vxstat,mpstat,lockstat -p 600 -i 10"
}

collectcmd() {
  OUT=$1
  CMD=$2
  TIMEX=$3

  if [ "$TIMEX" = "getruntime" ] ; then 
    ( timex $CMD > ${TIME}.${OUT}.out 2> ${TIME}.${OUT}.timex.out ; gzip ${TIME}.${OUT}.out ; gzip ${TIME}.${OUT}.timex.out ) &
  else
    ( $CMD > ${TIME}.${OUT}.out ; gzip ${TIME}.${OUT}.out ) &
  fi
}

collect() {
  if not isNumeric "$PERIOD" ; then
    fatalerror "PERIOD is not a number: $PERIOD"
  fi

  if not isNumeric "$INTERVAL" ; then
    fatalerror "INTERVAL is not a number: $INTERVAL"
  fi

  ITERATIONS=`echo "$PERIOD / $INTERVAL" | bc`
  debug "ITERATIONS $ITERATIONS"

  [ -d "${OUTDIR}/${DATE}" ] || mkdir "${OUTDIR}/${DATE}" || fatalerror "could not create output directory ${OUTDIR}/${DATE}"
  cd "${OUTDIR}/${DATE}"

  debug "Starting modules!"

  for module in `echo "$MODULES" | sed 's/,/ /g'` ; do
    case "$module" in
      "ps")
         debug $module
         [ $OS = "SunOS" -a $OSV = "9" ] && collectcmd "${module}" "ps -e -o user,group,pid,ppid,s,pmem,vsz,rss,pcpu,nlwp,time,etime,nice,pri,project,args"
         [ $OS = "SunOS" -a $OSV = "10" ] && collectcmd "${module}" "ps -e -o zone,user,group,pid,ppid,s,pmem,vsz,rss,pcpu,nlwp,time,etime,nice,pri,ctid,project,args" "getruntime"
         [ $OS = "AIX" ] && collectcmd "${module}" "ps -e -o user,group,pid,ppid,state,pmem,vsz,rssize,pcpu,thcount,time,etime,tpgsz,dpgsz,spgsz,shmpgsz,nice,pri,args"
         ;;
      "netstat-an")
         debug $module
         collectcmd "${module}" "netstat -an"
         ;;
      "df-k")
         debug $module 
         collectcmd "${module}" "df -k"
         ;;
      "iostat")
         debug $module
         filename="${module}_${INTERVAL}_${ITERATIONS}"
         if [ $OS = "SunOS" ] ; then
           collectcmd "$filename" "iostat -nTdYd $INTERVAL $ITERATIONS"
         elif [ $OS = "AIX" ] ; then
           collectcmd "$filename" "iostat -DTl $INTERVAL $ITERATIONS"
         else
           warning "$module is not supported in OS $OS"
         fi
         ;;
      "vmstat_page")
         debug $module 
         filename="${module}_${INTERVAL}_${ITERATIONS}"
         if [ $OS = "SunOS" ] ; then
           collectcmd "$filename" "vmstat -p $INTERVAL $ITERATIONS"
         else
           warning "$module is not supported in OS $OS"
         fi
         ;;
      "vmstat")
         debug $module 
         filename="${module}_${INTERVAL}_${ITERATIONS}"
         if [ $OS = "AIX" ] ; then
           collectcmd "$filename" "vmstat -wtIl $INTERVAL $ITERATIONS"
         else
           collectcmd "$filename" "vmstat $INTERVAL $ITERATIONS"
         fi
         ;;
      "vmstat_vmm")
         debug $module 
         if [ $OS = "AIX" ] ; then
           collectcmd "${module}" "vmstat -v"
         else
           warning "$module is not supported in OS $OS"
         fi
         ;;
      "mem_distr")
         debug $module
         filename="${module}_${INTERVAL}_${ITERATIONS}"
         if [ $OS = "AIX" ] ; then
           collectcmd "$filename" "svmon -G -O timestamp=on -i $INTERVAL $ITERATIONS"
         else
           warning "$module is not supported in OS $OS"
         fi
         ;;
      "vxstat")
         debug $module
         filename="${module}-uk-i${INTERVAL}-c${ITERATIONS}-S"
         [ `vxdg -q list | wc -l` -gt 0 ] &&  collectcmd "$filename" "vxstat -o alldgs -dv -u k -i $INTERVAL -c $ITERATIONS -S"
         ;;
      "vxdmp_ctlr")
         debug $module
         filename="${module}-s_interval${INTERVAL}_count${ITERATIONS}"
         collectcmd "$filename" "vxdmpadm -s iostat show groupby=ctlr interval=$INTERVAL count=$ITERATIONS"
         ;;
      "vxmemstat")
         debug $module
         collectcmd "$module" "vxmemstat"
         ;;
      "vxfsstat_cache" | "vxfsstat_shared" | "vxfsstat_per_fs")
         debug $module
         if not [ "$mntpts" ] ; then
           if [ $OS = "SunOS" ] ; then
             mntpts=`mount -p | awk '{if($4=="vxfs"){print $3}}'` 
           elif [ $OS = "AIX" ] ; then
             mntpts=`mount | awk '{if($3=="vxfs"){print $2}}'`
           else
             warning "$module is not supported in OS $OS"
             continue
           fi
           if not [ "$mntpts" ] ; then
             debug "No vxfs mounted"
             continue
           fi
           firstmp=`echo "$mntpts" | head -1`
           strfirstmp=`echo ${firstmp} | sed -e 's/\///' -e 's/\//_/g'`
         fi
         case "$module" in
           "vxfsstat_cache")
             collectcmd "$module-t${INTERVAL}-c${ITERATIONS}_${strfirstmp}" "vxfsstat -bi -t ${INTERVAL} -c ${ITERATIONS} ${firstmp}"
             ;;
           "vxfsstat_shared")
             collectcmd "$module-t${INTERVAL}-c${ITERATIONS}_${strfirstmp}" "vxfsstat -v -t ${INTERVAL} -c ${ITERATIONS} ${firstmp}"
             ;;
           "vxfsstat_per_fs")
             for mp in $mntpts ; do
               strmp=`echo ${mp} | sed -e 's/\///' -e 's/\//_/g'`
               collectcmd "${module}-t${INTERVAL}-c${ITERATIONS}_${strmp}" "vxfsstat -x -t ${INTERVAL} -c ${ITERATIONS} ${mp}"
             done
             ;;
         esac
         ;;
      "mpstat")
         debug $module
         filename="${module}_${INTERVAL}_${ITERATIONS}"
         collectcmd "$filename" "mpstat $INTERVAL $ITERATIONS"
         ;;
      "prstat-m")
         debug $module
         filename="${module}_${INTERVAL}_${ITERATIONS}"
         if [ $OS = "SunOS" ] ; then
           collectcmd "$filename" "prstat -mcn 1000 $INTERVAL $ITERATIONS"
         else
           warning "$module is not supported in OS $OS"
         fi
         ;;
      "lockstat")
         debug $module
         filename="${module}_sleep_5"
#         if [ $OS = "SunOS" -a $OSV = "10" ] ; then
          if [ $OS = "SunOS"  ] ; then
           collectcmd "$filename" "lockstat sleep 5"
         else
           warning "$module is not supported in OS $OS Version $OSV"
         fi
         ;;
      "swap_ph")
         debug $module
         filename="${module}"
         if [ $OS = "SunOS" ] ; then
           collectcmd "$filename" "swap -l"
         elif [ $OS = "AIX" ] ; then
           collectcmd "$filename" "lsps -a"
         else
           warning "$module is not supported in OS $OS"
         fi
         ;;
      "fcstat")
         debug $module
         if [ $OS = "AIX" ] ; then
           lsdev -l 'fcs*' -S a -F name | while read adapter ; do
             filename="${module}_${adapter}"
             collectcmd "$filename" "fcstat $adapter"
           done
         else
           warning "$module is not supported in OS $OS"
         fi
         ;;
      "who")
         debug $module
         collectcmd "$module" "who -u"
         ;;
      "ipcs")
         debug $module
         if [ $OS = "SunOS" ] ; then
           collectcmd "$module" "ipcs -A"
         elif [ $OS = "AIX" ] ; then
           collectcmd "$module" "ipcs -a"
         else
           warning "$module is not supported in OS $OS"
         fi
         ;;
      *) fatalerror "Wrong module: $module" ;;
    esac
  done

  debug "Modules started!"
}

clean() {
  if isNumeric "$KEEPDATA" ; then
    find "$OUTDIR" -type d -name '20[0-9][0-9].[0-1][0-9].[0-3][0-9]' -mtime +$KEEPDATA | while read daydir ; do
      find "$daydir" -type f -name '[0-2][0-9]*.out*' -mtime +$KEEPDATA -exec rm {} \;
      rmdir "$daydir"
    done
    debug "cleaning"
  else
    fatalerror "KEEPDATA is not a number: $KEEPDATA"
  fi
}

#
# Add VRTS path
#

if [ -d /opt/VRTS/bin ] ; then
  PATH=$PATH:/opt/VRTS/bin
  export PATH
fi

#
# global variables initialization
#

DEBUG="false"
ACTION=""
MODULES=""
PERIOD=600
INTERVAL=10
OUTDIR="/var/adm/hlar"
KEEPDATA=7
ITERATIONS=1
OS=`uname`
OSV=""
DATE=`date +%Y.%m.%d`
TIME=`date +%H.%M.%S`

# parse opts
while getopts :a:m:p:i:o:k:dh option
do
  case $option in
    a) ACTION=$OPTARG ;;
    m) MODULES=$OPTARG ;;
    p) PERIOD=$OPTARG ;;
    i) INTERVAL=$OPTARG ;;
    o) OUTDIR=$OPTARG ;;
    k) KEEPDATA=$OPTARG ;;
    d) DEBUG="true" ;;
    h) usage ; exit 0 ;;
    *) fatalerror "Invalid option found!" ;;
  esac
done

debug "DATE $DATE"
debug "TIME $TIME"
debug "ACTION $ACTION"
debug "MODULES $MODULES"
debug "PERIOD $PERIOD"
debug "INTERVAL $INTERVAL"
debug "OUTDIR $OUTDIR"
debug "KEEPDATA $KEEPDATA"
debug "OS $OS"

# OS version
case $OS in
  "SunOS")
    OSV=`uname -r | cut -f 2 -d .`
    ;;
  "AIX")
    OSV=`oslevel | cut -c -3`
    ;;
  *) fatalerror "OS $OS is not supported!" ;;
esac 

debug "OSV $OSV"

# is output directory exist?
[ -d "$OUTDIR" ] || fatalerror "Output directory $OUTDIR is not exist. Please create or set alternative (-o DIR)."

case $ACTION in
  collect) collect ;;
  clean) clean ;;
  *) fatalerror "Wrong action!" ;;
esac
